
local L = LibStub("AceLocale-3.0"):GetLocale("GridStatusPalaBeacon", true)



local GridStatusPalaBeacon = Grid:GetModule("GridStatus"):NewModule("GridStatusPalaBeacon", "AceTimer-3.0")

GridStatusPalaBeacon.menuName = L["PALA_BEACONS"];


local refreshTimer
local timerScheduled = false

local beaconInfo = GetSpellInfo(53563);



GridStatusPalaBeacon.options = false

GridStatusPalaBeacon.defaultDB = {

	alert_mybeacon = {
		enable = true,
		color = { r = 0, g = 1, b = 0, a = 1 },
		color1 = { r = 1, g = 1, b = 0, a = 1 },
		color2 = { r = 1, g = 0, b = 0, a = 1 },
		threshold1 = 30,
		threshold2 = 10,
		priority = 50,
		range = false,
		text = "B+",
		textnum = false,
		textnum1 = false,
		textnum2 = true,
	},

	alert_theirbeacon = {
		enable = true,
		color = { r = 0, g = 0, b = 1, a = 1 },
		priority = 40,
		range = false,
		text = "B-",
	}
}

local opMy = {

	["text"] = {
		type = "input",
		name = L["TEXT"],
		desc = L["STATUS_TEXT"],
		get = function() return GridStatusPalaBeacon.db.profile.alert_mybeacon.text end,
		set = function(_, v) GridStatusPalaBeacon.db.profile.alert_mybeacon.text = v end,
	},


	["textnum"] = {
		type = "toggle",
		name = (L["SHOW_TIME"]),
		get = function()
		    return GridStatusPalaBeacon.db.profile.alert_mybeacon.textnum
		end,
		set = function(_, v)
		    GridStatusPalaBeacon.db.profile.alert_mybeacon.textnum = v
		end,
	},	

	["tr1"] = {
		type = "group",
		name = L["THRESHOLD_1"],
		args = {

			["color1"] = {
				type = "color",
				name = L["COLOR"],
				hasAlpha = true,
				get = function ()
					local color = GridStatusPalaBeacon.db.profile.alert_mybeacon.color1
					return color.r, color.g, color.b, color.a
				end,
				set = function (_, r, g, b, a)
					local color = GridStatusPalaBeacon.db.profile.alert_mybeacon.color1
					color.r = r
					color.g = g
					color.b = b
					color.a = a or 1
				end,
			},
		
			["threshold1"] = {
				type = "range",
				name = L["THRESHOLD"],
				max = 300,
				min = 1,
				step = 1,
				get = function ()
					return GridStatusPalaBeacon.db.profile.alert_mybeacon.threshold1
				end,
				set = function (_, v)
					GridStatusPalaBeacon.db.profile.alert_mybeacon.threshold1 = v
				end,
			},

			["textnum2"] = {
				type = "toggle",
				name = (L["SHOW_TIME"]),
				desc = (L["SHOW_TIME_DESC"]),
				get = function()
					return GridStatusPalaBeacon.db.profile.alert_mybeacon.textnum1
				end,
				set = function(_, v)
					GridStatusPalaBeacon.db.profile.alert_mybeacon.textnum1 = v
				end,
			},	

		},
	},


	["tr2"] = {
		type = "group",
		name = L["THRESHOLD_2"],
		args = {

			["color2"] = {
				type = "color",
				name = L["COLOR"],
				hasAlpha = true,
				get = function ()
					local color = GridStatusPalaBeacon.db.profile.alert_mybeacon.color2
					return color.r, color.g, color.b, color.a
				end,
				set = function (_, r, g, b, a)
					local color = GridStatusPalaBeacon.db.profile.alert_mybeacon.color2
					color.r = r
					color.g = g
					color.b = b
					color.a = a or 1
				end,
			},
		
			["threshold2"] = {
				type = "range",
				name = L["THRESHOLD"],
				max = 300,
				min = 1,
				step = 1,
				get = function ()
					return GridStatusPalaBeacon.db.profile.alert_mybeacon.threshold2
				end,
				set = function (_, v)
					GridStatusPalaBeacon.db.profile.alert_mybeacon.threshold2 = v
				end,
			},
		
			["textnum2"] = {
				type = "toggle",
				name = (L["SHOW_TIME"]),
				desc = (L["SHOW_TIME_DESC"]),
				get = function()
					return GridStatusPalaBeacon.db.profile.alert_mybeacon.textnum2
				end,
				set = function(_, v)
					GridStatusPalaBeacon.db.profile.alert_mybeacon.textnum2 = v
				end,
			},	
		},
	},



}

local opTheir = {
	["text"] = {
		type = "input",
		name = L["TEXT"],
		desc = L["STATUS_TEXT"],
		get = function() return GridStatusPalaBeacon.db.profile.alert_theirbeacon.text end,
		set = function(_, v) GridStatusPalaBeacon.db.profile.alert_theirbeacon.text = v end,
	},

}


function GridStatusPalaBeacon:OnInitialize()
	self.super.OnInitialize(self)

	self:RegisterStatus("alert_mybeacon", L["MY_STATUS_NAME"], opMy, false)
	self:RegisterStatus("alert_theirbeacon", L["THEIR_STATUS_NAME"], opTheir, false)
end

function GridStatusPalaBeacon:OnEnable()
	self.super.OnEnable(self)
end

function GridStatusPalaBeacon:OnStatusEnable(status)
	if (status == "alert_mybeacon") or (status == "alert_theirbeacon") then
		if not timerScheduled then 
			refreshTimer = self:ScheduleRepeatingTimer("UpdateAllUnits", 0.3)	
			timerScheduled = true
		end
		self:UpdateAllUnits()
	end
end

function GridStatusPalaBeacon:OnStatusDisable(status)
	if (status == "alert_mybeacon") or (status == "alert_theirbeacon") then
		if not (self.db.profile.alert_mybeacon.enable or self.db.profile.alert_theirbeacon.enable) then
			if timerScheduled then
				timerScheduled = false
				self:CancelTimer(refreshTimer)
			end
		end
		self.core:SendStatusLostAllUnits(status)
	end
end

function GridStatusPalaBeacon:UpdateAllUnits()
	for guid, unitid in Grid:GetModule("GridRoster"):IterateRoster() do
		self:UpdateUnit("UpdateAllUnits", unitid)
	end
end

function GridStatusPalaBeacon:UpdateUnit(event, unit)

	local myOpt = self.db.profile.alert_mybeacon
	local theirOpt = self.db.profile.alert_theirbeacon
	local now = GetTime()

	local myDetected = false
	local theirDetected = false

	for i=1,999 do
		local bname,brank,bicon,bcount,btype,bdur,bexptime,bsource = UnitBuff(unit, i)
		if not bname then break end
		local btime = bexptime - now
		if bname == beaconInfo then
		    local isMine = bsource and UnitIsUnit(bsource, "player")
			if self.db.profile.alert_mybeacon.enable and isMine and btime then
				myDetected = true	

				t1,t2,c1,c2 = myOpt.threshold1, myOpt.threshold2, myOpt.color1, myOpt.color2

				local tx1, tx2 = myOpt.textnum1, myOpt.textnum2

				local textNum = myOpt.textnum

				if t1 < t2 then
					t1,t2,c1,c2,tx1,tx2 = t2,t1,c2,c1,tx2,tx1
				end
			
				local c = myOpt.color

				if btime < t1 then 
					c = c1 
					textNum = tx1
				end

				if btime < t2 then 
					c = c2 
					textNum = tx2
				end
				

				local text = myOpt.text
				if textNum then text = tostring(math.floor(btime)) end


				self.core:SendStatusGained(UnitGUID(unit), "alert_mybeacon", myOpt.priority, (myOpt.range and 40), c, text, nil, nil, bicon, bexptime-bdur, bdur)
			end
			if self.db.profile.alert_theirbeacon.enable and btime and (not isMine) and (not theirDetected) then
				theirDetected = true	
				self.core:SendStatusGained(UnitGUID(unit), "alert_theirbeacon", theirOpt.priority, (theirOpt.range and 40), theirOpt.color, theirOpt.text)
			end
		end
	end
	
	if not myDetected then
		self.core:SendStatusLost(UnitGUID(unit), "alert_mybeacon")
	end

	if not theirDetected then
		self.core:SendStatusLost(UnitGUID(unit), "alert_theirbeacon")
	end

end
